package com.biganto.visual.roompark.presentation.screen.albums.util



import android.content.Context
import android.graphics.*
import android.view.Surface
import com.biganto.visual.roompark.R
import timber.log.Timber
import kotlin.math.tan


/**
 * Created by Vladislav Bogdashkin on 18.07.2019.
 */

private const val CONST_SLEEP_THRESHOLD = 8L
private const val CONST_HEIGHT_MARGIN_BORDER_PX = 8
private const val CONST_DECAY_TIMER = 500L

class RendererThread(val context: Context, val surface: SurfaceTexture)
    : Thread(){

    private val dens:Float by lazy { context.resources.displayMetrics.density }

    private val SLEEP_THRESHOLD get() = CONST_SLEEP_THRESHOLD

    private val HEIGHT_MARGIN_BORDER_PX get() =
                    (CONST_HEIGHT_MARGIN_BORDER_PX *dens).int

    private val DECAY_TIMER get() = CONST_DECAY_TIMER

    private var isDirty = true
    var isStopped = false

    private var width = 0
    private var height = 0

    private var centerX = 0

    private var holdIndex = 0
        get() = if (isDragging) field
        else currentIndex

    private var currentIndex:Int = 0

    set(value) {Timber.w("currentIndex to: $value");field = value}



    fun setIndex(index:Int){
        if (!isDragging) holdIndex = index
        currentIndex = index
        isDirty=true
    }


    private val Float.int:Int get() {return this.toInt()}

    fun setCenter(centerX:Int){
        this.centerX = centerX
        isDirty = true
    }

    private var scrollScaleFactor = 1f
    private var touchOffset = 0f
    private var scrollOffset = 0f
    private var isDragging = false
    private var slidePosition = 0f
    private var dragKoef = 1f

    fun setSlidePosition(newPosition:Float){
        slidePosition = newPosition
        isDirty = true
    }

    private var decayFraction  = 0f

    private fun timerDecay(mills:Long){
        isDirty=true
        decayFraction+=mills
        animateBorders()
        scrollOffset = scrollOffset.smoothLerp(touchOffset-scrollOffset,.2f)
        if (touchOffset> 0 && scrollOffset >= touchOffset) {
            scrollOffset = touchOffset
            isDirty=false
        }
        else if (touchOffset< 0 && scrollOffset <= touchOffset) {
            scrollOffset = touchOffset
            isDirty=false
        }
        if (decayFraction> DECAY_TIMER) isDirty=false
        val dirty = animateBorders()
        if (!isDirty) isDirty = dirty
    }

    private fun animateBorders():Boolean{
        if (isDragging && dragKoef>0){
            dragKoef = dragKoef.smoothLerp(-dragKoef,.12f)
            if (dragKoef<.1f) dragKoef = 0f
            return  dragKoef>0
        }

        if (!isDragging && dragKoef<1f) {
            dragKoef = dragKoef.smoothLerp(1f-dragKoef,.03f)
            if (dragKoef>.99f) dragKoef = 1f
            return dragKoef<1f
        }
        return false
    }

    private fun Float.smoothLerp( delta: Float, fraction:Float =.4f): Float {
        return  this + fraction * (delta)
    }

    fun setSize(w: Int, h: Int){
        width=w
        height=h
    }

    private val pinColor:Int by lazy {
        return@lazy context.resources.getColor(R.color.colorOpacityBackground)
    }

    private val trianglePath:Path
    get() {
        val p = Path()
        p.fillType = Path.FillType.EVEN_ODD
        p.moveTo(centerX.toFloat(),height.toFloat())
        p.lineTo(centerX.toFloat() + height*tan(Math.toRadians(60.0)).toFloat(),height.toFloat())
        p.lineTo(centerX.toFloat() - height*tan(Math.toRadians(60.0)).toFloat(),height.toFloat())
        p.lineTo(centerX.toFloat(),height.toFloat())
        p.close()
        return p
    }

    private val trianglePaint:Paint by lazy {
        val p = Paint()
        p.color = pinColor
        p.style = Paint.Style.FILL_AND_STROKE
        p.isAntiAlias = true
        p
    }

    override fun run() {
        super.run()
        Timber.w("isStopped: $isStopped")
        Timber.w("isDirty: $isDirty")
        isDirty = true

        while (!isStopped) {
            while (!isDirty)
                sleep(SLEEP_THRESHOLD) // in real life this sleep is more complicated
            isDirty = false

            val surf = Surface(surface)
            val dRect = Rect(
                0
                , 0
                , width
                , height
            )
            val c = surf.lockCanvas(dRect)
            c.drawColor(Color.TRANSPARENT)
            c.drawPath(trianglePath,trianglePaint)
            timerDecay(SLEEP_THRESHOLD)



            surf.unlockCanvasAndPost(c)
            surf.release()
//            isDirty = false
        }
        Timber.w("STOPPED; $isStopped")
        Timber.w("STOPPED; $isDirty")
        isDirty=false
    }
}

